import * as THREE from 'three';
import { useEffect } from 'react';
import { useProductStore } from '../../../store/simulation/useProductStore';
import { useSelectedProduct } from '../../../store/simulation/useSimulationStore';
import AddOrRemoveEventsInProducts from './events/addOrRemoveEventsInProducts';
import { upsertProductOrEventApi } from '../../../services/simulation/UpsertProductOrEventApi';
import { getAllProductsApi } from '../../../services/simulation/getallProductsApi';
import { useVehicleStore } from '../../../store/simulation/useVehicleStore';
import { useArmBotStore } from '../../../store/simulation/useArmBotStore';

function Products() {
    const { products, getProductById, addProduct, setProducts } = useProductStore();
    const { selectedProduct, setSelectedProduct } = useSelectedProduct();
    const { addVehicle, clearvehicles } = useVehicleStore();
    const { addArmBot, clearArmBots } = useArmBotStore();

    useEffect(() => {
        const email = localStorage.getItem('email')
        const organization = (email!.split("@")[1]).split(".")[0];
        getAllProductsApi(organization).then((data) => {
            if (data.length === 0) {
                const id = THREE.MathUtils.generateUUID();
                const name = 'Product 1';
                addProduct(name, id);
                upsertProductOrEventApi({ productName: name, productId: id, organization: organization })
                setSelectedProduct(id, name);
            } else {
                setProducts(data);
                setSelectedProduct(data[0].productId, data[0].productName);
            }
        })
    }, [])

    useEffect(() => {
        if (selectedProduct.productId) {
            const product = getProductById(selectedProduct.productId);
            if (product) {
                clearvehicles();
                product.eventDatas.forEach(events => {
                    if (events.type === 'vehicle') {
                        addVehicle(selectedProduct.productId, events);
                    }
                });
            }
        }
    }, [selectedProduct, products]);

    useEffect(() => {
        if (selectedProduct.productId) {
            const product = getProductById(selectedProduct.productId);
            if (product) {
                clearArmBots();
                product.eventDatas.forEach(events => {
                    if (events.type === 'roboticArm') {
                        addArmBot(selectedProduct.productId, events);
                    }
                });
            }
        }
    }, [selectedProduct, products]);

    return (
        <>

            <AddOrRemoveEventsInProducts />

        </>
    )
}

export default Products